function interp(x, v, f0)
    try
        return LinearInterpolation(x, v)(f0)
    catch err
        if isa(err, BoundsError)
            lower, upper = x[1], x[end]
            throw("f0 must be between $lower-$upper")
        end
    end
end
"""
    volt_to_pressure(15e6; hydrophone_id = :Onda_HGL0200_2322, preamp_id = :Onda_AH2020_1238_20dB)
    volt_to_pressure(15e6; hydrophone_id = :PA_3197)

# Arguments
- `f0`: Frequency in Hz.
- `hydrophone_id`: Currently accepts two: :Onda_HGL0200_2322, :PA_3197.
- `preamp_id`: Only for :Onda_HGL0200_2322. Choices are: :Onda_AH2020_1238_0dB, :Onda_AH2020_1238_20dB.
# Returns
- `factor`: conversion factor in form of pascal/volt.
"""
function volt_to_pressure(f0, hydrophone_id::Symbol)
    f0 = f0*u"Hz"
    calibration_data, _ = get_calibration(hydrophone_id)
    frequencies = calibration_data[:, 1] * u"MHz"
    if occursin("PA_", String(hydrophone_id))
        sensitivities = calibration_data[:, 2] * u"mV/MPa"
    else
        sensitivities = calibration_data[:, 2] * u"dBV/μPa"
    end
    sensitivity_at_f0 = interp(frequencies, sensitivities, f0)
    return calculate_pressure_per_volt(sensitivity_at_f0)
end

function calculate_pressure_per_volt(volt_per_pressure)
    volt_per_pressure = uconvertrp(u"V/Pa", volt_per_pressure)
    pressure_per_volt = 1/volt_per_pressure
    return pressure_per_volt
end


function volt_to_pressure(f0, hydrophone_id::Symbol, preamp_id::Symbol)
    factor, _ = volt_to_pressure_and_phase(f0, hydrophone_id, preamp_id)
    return factor
end

function volt_to_pressure_and_phase(f0, hydrophone_id::Symbol, preamp_id::Symbol)
    f0 = f0 * u"Hz"
    calibration_data, preamp_data = get_calibration(hydrophone_id, preamp_id)

    if isempty(preamp_data)
        error("No preamplifier calibration data found")
    end

    hydrophone_freqs = calibration_data[:, 1] * u"MHz"
    hydrophone_sensitivities = calibration_data[:, 2] * u"dBV/μPa"
    hydrophone_capacitances = calibration_data[:, 5] * u"pF"
    hydrophone_sensitivity_at_f0 = interp(hydrophone_freqs, hydrophone_sensitivities, f0)
    hydrophone_capacitance_at_f0 = interp(hydrophone_freqs, hydrophone_capacitances, f0)

    preamp_freqs = preamp_data[:,1] * u"MHz"
    preamp_gains = uconvertrp.(NoUnits, preamp_data[:, 2] * u"dB")
    preamp_phases = preamp_data[:, 3]
    preamp_capacitances = preamp_data[:, 4] * u"pF"
    preamp_gain_at_f0 = interp(preamp_freqs, preamp_gains, f0)
    preamp_phase_at_f0 = interp(preamp_freqs, preamp_phases, f0)
    preamp_capacitance_at_f0 = interp(preamp_freqs, preamp_capacitances, f0)

    voltage_factor = hydrophone_capacitance_at_f0 ./ (hydrophone_capacitance_at_f0 + preamp_capacitance_at_f0)

    # Combine result
    combined_sensitivity = preamp_gain_at_f0 * hydrophone_sensitivity_at_f0 * voltage_factor
    return calculate_pressure_per_volt(combined_sensitivity), preamp_phase_at_f0
end
